using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing.Design;
using Dapfor.Net.Data;
using Dapfor.Net.Grid.Tutorial.Editors;

namespace Dapfor.Net.Grid.Tutorial.BusinessData.OrderBook
{
    public class Strategy : INotifyPropertyChanged
    {
        #region Fields

        private readonly DateTime _creationDate;
        private readonly string _id;
        private readonly ThreadSafeBindingList<Order> _orders = new ThreadSafeBindingList<Order>();
        private DateTime _lastChangeTimeStamp;

        #endregion

        public Strategy(string id)
        {
            _id = id;
            _creationDate = _lastChangeTimeStamp = DateTime.Now;

            _orders.ListChanged += delegate
            {
                _lastChangeTimeStamp = DateTime.Now;
                if(PropertyChanged != null)
                {
                    PropertyChanged(this, new PropertyChangedEventArgs("LastChangeTimeStamp"));
                    PropertyChanged(this, new PropertyChangedEventArgs("Status"));
                }
            };
        }

        #region Public properties

        [HierarchicalField]
        public IList<Order> Orders
        {
            get { return _orders; }
        }

        [Field("TimeStamp")]
        public DateTime CreationDate
        {
            get { return _creationDate; }
        }

        public DateTime LastChangeTimeStamp
        {
            get { return _lastChangeTimeStamp; }
        }

        public string Id
        {
            get { return _id; }
        }

        [Editor(typeof(StrategyStatusEditor), typeof(UITypeEditor))]
        public Status Status
        {
            get
            {
                Status status = Status.Created;
                foreach (Order order in _orders)
                {
                    switch (order.Status)
                    {
                        case Status.Cancelled:
                            if (!Equals(status, Status.Active))
                            {
                                status = Status.Cancelled;
                            }
                            break;

                        case Status.Active:
                            status = Status.Active;
                            break;
                    }
                }
                return status;
            }
            set{}
        }

        #endregion


        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion
    }
}